;=============================================================
; Bomb Koopa
; Description: A koopa which, when stomped, explodes after a
; certain amount of time.
;
; Uses extra bits: NO
;
; Uses extra bytes: YES
; Extra byte 1:
;   80 - Stay on ledges
;   40 - Follow Mario
;   C0 - Both of those
;=============================================================
; By Erik557
; Inspired by mikeyk's variant, but recoded from scratch.
;=============================================================

!ShellessKoopaNumber = $03  ;   sprite number for thomas' shell-less koopa
!TimeBefFlashing     = $30  ;   time before the shell starts flashing
!TimeBefExplode      = $10  ;   time before the shell actually explodes

AngrySpeeds:
       db $38,$C8

;=========================
; INIT and MAIN Wrappers
;=========================

print "INIT ",pc
       JSL SMW_GetRand|!bankB      ;\  Get starting frame
       STA !1570,x                 ;/
       JSL SubHorzPos              ;\
       TYA                         ; | Face player
       STA !157C,x                 ;/
       LDA !7FAB40,x               ;\  Save extra property bytes
       STA !1594,x                 ;/
       RTL

print "MAIN ",pc
       PHB
       PHK
       PLB
       INC !1570,x          ;   Increase sprite's frame counter
       LDA !14C8,x
       CMP #$09             ;\  Use special code if stomped
       BNE +                ; |
       JSR Stomped          ;/
       PLB
       RTL
+      CMP #$0A             ;\  Don't execute any code if in a kicked state
       BCS +                ;/
       JSR ShadyKoopa       ;   Jump to the main routine
+      PLB
       RTL

XSpeeds:
      db $08,$F8

;===============
; Main routine
;===============

Stomped:
       LDA #$82
       STA !167A,x
       LDA #$0B
       STA !1686,x
       INC !1FE2,x
       LDY !15EA,x
       LDA #$F0
       STA $0301|!addr,y
       STA $0305|!addr,y
       LDA $9D
       BNE +
       LDA !1540,x
       BEQ .clear
       CMP #$FF
       BEQ .set
       CMP #$06
       BEQ .die
       CMP.b #!TimeBefExplode
       BCS +
       LDA !1570,x
       LSR #3
       AND #$01
       BEQ +
       LDA $030B|!addr,y
       EOR #$02
       STA $030B|!addr,y
+      RTS

.clear
       LDA #$02
       STA !1686,x
       STZ !167A,x
       RTS

.set
       LDA.b #!TimeBefFlashing-1   ;   minus 1 to account for the frame lost
       STA !1540,x
       RTS

.die
       LDA !1FD6,x
       BEQ +
       CMP #$02
       BEQ .settingDone
       JSL SubHorzPos
       TYA
       STA !157C,x
       LDA #$09
       STA $1DFC|!addr
       INC !1FD6,x
.settingDone
       LDY !157C,x
       LDA AngrySpeeds,y
       STA !B6,x
       JSL SMW_SpriteSpriteInteraction|!bankB
       JSL SMW_UpdateSpritePosWithGravity|!bankB
       JSL SMW_SpriteMarioInteraction|!bankB
       BCC .noint
       JSL SMW_HurtMario|!bankB
.noint
       LDA !1570,x
       LSR #2
       AND #$07
       LDY !15EA,x
       TAX
       LDA FlashPals,x
       STA $030B|!addr,y
       LDX $15E9|!addr
       LDA #$07
       STA !1540,x
       RTS

FlashPals:
       db $A0,$A2,$A4,$A6,$A8,$AA,$AC,$AE

+      LDA #$07
       STA !9E,x
       LDA #$0A
       STA !14C8,x
       JSL SMW_InitSpriteTables|!bankB
       INC !187B,x
       RTS

ShadyKoopa:
       LDA !14C8,x                 ;\
       EOR #$08                    ; | if not alive, return
       ORA $9D                     ; | if sprites locked, return
       BEQ +                       ; |
       JMP .LockedSprites          ;/

+      JSL SubOffScreenA           ;   a should be 0
       LDA !1570,x                 ;\
       LSR #3                      ; | set animation frame
       AND #$01                    ; |
       STA !1602,x                 ;/

       JSL SMW_SpriteSpriteInteraction|!bankB
       JSL SMW_UpdateSpritePosWithGravity|!bankB

       LDY !157C,x                 ;\
       LDA XSpeeds,y               ; | Check if the sprite is on a slope
       EOR !15B8,x                 ; |
       ASL                         ;/
       LDA XSpeeds,y               ;
       BCC .NotOnSlope             ; If not, branch
       CLC                         ;
       ADC !15B8,x                 ;
.NotOnSlope                        ;
       STA !B6,x                   ; Set X speed

       LDA !1588,x                 ;\
       AND #$04                    ; | branch if not on ground
       BEQ .InAir                  ;/

       LDA !1588,x                 ;\ branch if touching a layer 2 ceiling
       BMI .air                    ;/
       LDA #$00
       LDY !15B8,x                 ;\ branch if not on a slope
       BEQ +                       ;/
.air
       LDA #$18
+      STA !AA,x                   ; set y speed
       STZ !151C,x
       BRA .XSpdDone

.InAir
       BIT !1594,x
       BPL .XSpdDone
       LDA !151C,x
       BNE .XSpdDone
       STZ !1570,x                 ; stop animations
       JSR TurnSprite              ; flip
       LDA #$01
       STA !151C,x

.XSpdDone
       BIT !1594,x
       BVC +
       LDA !1570,x
       AND #$7F
       BNE +
       LDA !157C,x
       PHA
       JSL SubHorzPos
       STY $00
       PLA
       CMP $00
       BEQ +
       JSR TurnSprite
       
+      LDA !157C,x                 ;\
       INC                         ; |
       AND !1588,x                 ; | return if touching an object on
       AND #$03                    ; | the direction Mario is moving
       BEQ .LockedSprites          ; |
       JSR TurnSprite              ;/
.LockedSprites
       PEA.w ($01)|(.drawn>>16<<8) ;\
       PLB                         ; |
       PHK                         ; |\
       PEA.w .drawn-1              ; | | jsl to 018bc3
       PEA $80C9                   ; | |
       JML $018BC3|!bankB          ; |/
.drawn                             ;/
       PLB                         ; restore data bank

       LDA !154C,x
       ORA $9D
       BNE .Return3
       JSR CapeInteraction
       JSL SMW_SpriteMarioInteraction|!bankB
       LDA !14C8,x
       CMP #$09
       BNE .Return3
       LDA.b #!TimeBefFlashing
       STA !1540,x
SpawnShellless:
       LDY.b #!SprSize-1
-      LDA !14C8,y
       BEQ getReady
       DEY
       BPL -
ShadyKoopa_Return3:
       RTS

getReady:
       LDA #$01
       STA !14C8,y
       TYX
       LDA.b #!ShellessKoopaNumber
       STA !7FAB9E,x
       JSL SMW_InitSpriteTables|!bankB
       LDA #$08
       STA !7FAB10,x
       JSL.l read3($0187A8)|!bankB
       LDX $15E9|!addr
       LDA !E4,x
       STA.w !E4,y
       LDA !14E0,x
       STA !14E0,y
       LDA !D8,x
       STA.w !D8,y
       LDA !14D4,x
       STA !14D4,y
       LDA !164A,x
       STA !164A,y
       LDA #$10
       STA !1564,y
       PHY
       JSL SubHorzPos
       LDA dumbTable,y
       STY $00
       PLY
       STA.w !B6,y
       LDA $00
       EOR #$01
       STA !157C,y
       LDA #$10
       STA !154C,y
       STA !1528,y
       RTS

dumbTable:
       db $C0,$40

;========================
; Misc. subroutines
;========================

;==================
  TurnSprite:
;==================

       LDA !15AC,x          ;\ don't turn is this is already set
       BNE .NoTurn          ;/
       LDA #$08             
       STA !15AC,x
       LDA !B6,x
       EOR #$FF
       INC
       STA !B6,x
       LDA !157C,x
       EOR #$01
       STA !157C,x
.NoTurn
       RTS

;==================
CapeInteraction:
;==================

       LDA $13E8|!addr
       BEQ .return
       LDA !15D0,x
       ORA !1FE2,x
       BNE .return
       LDA !1632,x
       PHY
       LDY $74
       BEQ +
       EOR #$01
+      PLY
       EOR $13F9|!addr
       BNE .return
       JSL $03B69F|!bankB
       LDA $13E9|!addr
       SEC
       SBC #$02
       STA $00
       LDA $13EA|!addr
       SBC #$00
       STA $08
       LDA #$14
       STA $02
       LDA $13EB|!addr
       STA $01
       LDA $13EC|!addr
       STA $09
       LDA #$10
       STA $03
       JSL $03B72B|!bankB
       BCC .return
       LDA #$03
       STA $1DF9|!addr
       TDC
       JSL $02ACE5|!bankB
       LDA #$09
       STA !14C8,x
       LDA.b #!TimeBefFlashing
       STA !1540,x
       INC !1FD6,x
       LDA #$B0
       STA !AA,x
       JSL SubHorzPos
       LDA SpunXSpeeds,y
       STA !B6,x
       TYA
       EOR #$01
       STA !157C,x
       LDA !15F6,x
       ORA #$80
       STA !15F6,x
.return
       RTS

SpunXSpeeds:
       db $F8,$08
